export default function ($ = jQuery, Drupal, once, dropZones, Sortable) {

  // Drag to reorder a block.
  // This is a reimplementation of layout-builder.js layoutBuilderBlockDrag.
  Drupal.behaviors.LBPlusSortBlockDrag = {
    attach(context) {
      Array.prototype.forEach.call(
        context.querySelectorAll('.layout__region'),
        (region) => {
          Sortable.create(region, {
            draggable: '.layout-builder-block',
            ghostClass: 'ui-state-drop',
            group: 'builder-region',
            onEnd: (event) => {
              const $item = $(event.item);
              const $from = $(event.from);

              // Check if the region from the event and region for the item match.
              const itemRegion = $item.closest('.layout__region');
              if (event.to === itemRegion[0]) {
                // Find the destination delta.
                const deltaTo = $item.closest('[data-layout-delta]').data('layout-delta');
                // If the block didn't leave the original delta use the destination.
                const deltaFrom = $from
                  ? $from.closest('[data-layout-delta]').data('layout-delta')
                  : deltaTo;
                Drupal.ajax({
                  url: [
                    $item.closest('[data-layout-update-url]').data('layout-update-url'),
                    deltaFrom,
                    deltaTo,
                    itemRegion[0].getAttribute('region'),
                    $item.data('layout-block-uuid'),
                    $item.prev('[data-layout-block-uuid]').data('layout-block-uuid'),
                  ]
                    .filter((element) => element !== undefined)
                    .join('/'),
                }).execute();
              }
            },
          });
        },
      );
    },
  };

  // Drag to choose a block.
  Drupal.behaviors.LBPlusAddBlockDrag = {
    attach: function (context, settings) {
      once('addBlockDrag', '.tabbed-content .draggable-block', context).forEach(draggableBlock => {
        draggableBlock.ondrag = e => {
          dropZones.addSectionDropZones(null, Drupal.t('Add a new section'));
          dropZones.addRegionDropZones();
          // Hide the blank page instructions.
          const blankPage = document.getElementById('lb-plus-blank-page');
          if (blankPage) {
            blankPage.style.display = 'none';
          }
        };
        draggableBlock.ondragstart = e => {
          e.dataTransfer.setData('text/json', JSON.stringify({
            id: e.target.id,
            type: 'block',
          }));
          e.dataTransfer.setData('block', 'true');
        };
        draggableBlock.ondragend = e => {
          dropZones.clearDropZones();
          // Reveal the blank page instructions if the block wasn't placed.
          const blankPage = document.getElementById('lb-plus-blank-page');
          if (blankPage) {
            blankPage.style.display = 'block';
          }
        };
      });
    },
  };

  // Click the open sidebar link for the user on blank pages.
  Drupal.behaviors.LBPlusClickAddBlock = {
    attach: (context, settings) => {
      once('clickAddBlock', '.blank-page', context).forEach(addBlockLink => {
        addBlockLink.click();
      });
    },
  };

}

// A block was dropped.
document.addEventListener('element-dropped', e => {
  if (e.detail.type === 'block') {
    const draggedBlock = document.getElementById(e.detail.id);
    const dropZone = e.detail.dropEvent.target;
    const type = dropZone.getAttribute('drop-zone-type');
    let destination = {
      type: type,
    };

    if (type === 'region') {
      destination = {
        ...destination,
        section: dropZone.closest('.lb-plus-section').id,
        position: dropZone.getAttribute('position'),
        region: dropZone.closest('.layout__region').getAttribute('region'),
      };
    }
    if (type === 'section') {
      // Pass either the section ID to put this block in front of, or the string
      // "last" to put it at the end.
      destination.section = dropZone.nextElementSibling?.classList.contains('lb-plus-section') ? dropZone.nextElementSibling.id : 'last';
    }

    // Remove the blank page instructions if the block was placed.
    const blankPage = document.getElementById('lb-plus-blank-page');
    if (blankPage) {
      blankPage.remove();
    }

    // Place the block.
    let ajaxConfig = {
      url: '/lb-plus/place-block/' + drupalSettings['LB+'].sectionStorageType + '/' + drupalSettings['LB+'].sectionStorage,
      type: 'POST',
      dataType: 'text',
      progress: {
        type: 'fullscreen',
        message: Drupal.t('Saving section order...')
      },
      submit: {
        place_block: {
          plugin_id: draggedBlock.id,
          destination: destination,
        },
      },
      error: (xhr, status, error) => {
        // Remove the AJAX progress spinner.
        document.querySelectorAll('.ajax-progress').forEach(progress => {
          progress.remove();
        });
        // Display a message.
        const messages = new Drupal.Message();
        messages.add('<span id="place-block-error">Unable to place the block.</span>', {type: 'error'});
        document.getElementById('place-block-error').scrollIntoView({behavior: 'smooth', block: 'start'});
      },
    };
    let ajax = Drupal.ajax(ajaxConfig);
    ajax.execute();
  }
});
