export default function ($ = jQuery, Drupal, once, dropZones) {

  // Sort handle icon.
  Drupal.behaviors.LBPlusDraggableSectionHandles = {
    attach: (context, settings) => {
      once('draggableSectionHandles', '.lb-plus-section-admin .sort', context).forEach(sortHandle => {
        sortHandle.onmousedown = e => {
          // The drag handle was clicked, set the section to draggable.
          let draggedSection = e.target.closest('.lb-plus-section');
          draggedSection.setAttribute('draggable', 'true');
        }
      });
    }
  };

  // Draggable/sortable sections.
  Drupal.behaviors.LBPlusDraggableSections = {
    attach: (context, settings) => {
      once('draggableSections', '.lb-plus-section', context).forEach(draggableSection => {
        draggableSection.ondrag = e => {
          dropZones.addSectionDropZones(e.target);
        };
        draggableSection.ondragstart = e => {
          // Store the element ID for placing once the element has been dropped.
          e.dataTransfer.setData('text/json', JSON.stringify({
            id: e.target.id,
            type: 'section',
          }));
          let draggedSection = e.target;
          // Disable hovers on other sections.
          getSections().forEach(draggableSection => {
            if (draggableSection !== draggedSection) {
              draggableSection.classList.remove('hover');
            }
          });
        };
        draggableSection.ondragend = e => {
          dropZones.clearDropZones();
          // The sort icon set this section to draggable. Return it to not
          // draggable now that the section has been dropped so that text can be
          // selected etc.
          e.target.setAttribute('draggable', 'false');
          // Enable section hovers.
          getSections().forEach(draggableSection => {
            if (!draggableSection.classList.contains('hover')) {
              draggableSection.classList.add('hover');
            }
          });
        };
      });
    },
  };

  // A section was dropped.
  document.addEventListener('element-dropped', e => {
    if (e.detail.type === 'section') {
      // Get the original order of the sections in case this operation fails.
      let originalSectionsOrder = [];
      getSections().forEach(section => {
        originalSectionsOrder.push(section.id);
      });
      // Move the section.
      const draggedSection = document.getElementById(e.detail.id);
      const dropZone = e.detail.dropEvent.target;
      dropZone.parentElement.insertBefore(draggedSection, dropZone);
      // Get the new order of the sections.
      let sectionsOrder = [];
      getSections().forEach(section => {
        sectionsOrder.push(section.id);
      });

      // Update the section storage.
      let ajaxConfig = {
        url: '/lb-plus/move-section/' + drupalSettings['LB+'].sectionStorageType + '/' + drupalSettings['LB+'].sectionStorage,
        type: 'POST',
        dataType: 'text',
        progress: {
          type: 'fullscreen',
          message: Drupal.t('Saving section order...')
        },
        submit: {sections_order: sectionsOrder},
        error: (xhr, status, error) => {
          originalSectionsOrder.forEach(sectionId => {
            // Updating failed, put the sections back how they were before.
            const section = document.getElementById(sectionId);
            section.parentElement.appendChild(section);
          });
          // Remove the AJAX progress spinner.
          document.querySelectorAll('.ajax-progress').forEach(progress => {
            progress.remove();
          });
          // Display a message.
          const messages = new Drupal.Message();
          messages.add('<span id="update-error">Unable to save the section order.</span>', {type: 'error'});
          document.getElementById('update-error').scrollIntoView({behavior: 'smooth', block: 'start'});
        },
      };
      let ajax = Drupal.ajax(ajaxConfig);
      ajax.execute();
    }
  });

}

export const getSections = () => {
  return document.querySelectorAll('.lb-plus-section');
};

// Scroll up and down when items are dragged to the top and bottom of the page.
jQuery(document).ready(() => {
  jQuery().dndPageScroll({
    delay: 10,
    height: 50,
  });
});

// Clicks the configure link on a section.
jQuery.fn.LBPlusConfigureSection = (uuid) => {
  jQuery('#' + uuid).find('.configure-link').click();
};
