<?php

namespace Drupal\lb_plus\Controller;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\layout_builder\SectionComponent;
use Drupal\layout_builder\SectionStorageInterface;
use Drupal\layout_builder\Plugin\Block\InlineBlock;
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Layout\LayoutPluginManagerInterface;
use Drupal\layout_builder\Controller\LayoutRebuildTrait;
use Drupal\layout_builder\LayoutTempstoreRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * Returns responses for Layout Builder Plus routes.
 */
class DuplicateBlock extends ControllerBase {

  use LayoutRebuildTrait;

  protected UuidInterface $uuid;
  protected LayoutPluginManagerInterface $layout;
  protected EventDispatcherInterface $eventDispatcher;
  protected PluginManagerInterface $pluginManagerBlock;
  protected LayoutTempstoreRepositoryInterface $tempstoreRepository;

  public function __construct(LayoutTempstoreRepositoryInterface $tempstore_repository, LayoutPluginManagerInterface $layout, PluginManagerInterface $plugin_manager_block, EventDispatcherInterface $event_dispatcher, UuidInterface $uuid) {
    $this->tempstoreRepository = $tempstore_repository;
    $this->pluginManagerBlock = $plugin_manager_block;
    $this->eventDispatcher = $event_dispatcher;
    $this->layout = $layout;
    $this->uuid = $uuid;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('layout_builder.tempstore_repository'),
      $container->get('plugin.manager.core.layout'),
      $container->get('plugin.manager.block'),
      $container->get('event_dispatcher'),
      $container->get('uuid')
    );
  }

  /**
   * Builds the response.
   */
  public function duplicate(SectionStorageInterface $section_storage = NULL, $delta = NULL, $region = NULL, $uuid = NULL) {
    $section = $section_storage->getSection($delta);
    $component = $section->getComponent($uuid);
    $block_plugin = $component->getPlugin();
    $configuration = $block_plugin->getConfiguration();

    if ($block_plugin instanceof InlineBlock) {
      // Create a block_content clone.
      if (!empty($configuration['block_serialized'])) {
        $block_content = unserialize($configuration['block_serialized']);
      } elseif (!empty($configuration['block_revision_id'])) {
        $block_content = $this->entityTypeManager()->getStorage('block_content')->loadRevision($configuration['block_revision_id']);
      }
      $cloned_block_content = $block_content->createDuplicate();
      $configuration['block_serialized'] = serialize($cloned_block_content);
    }

    // Add the cloned block next to the original.
    $cloned_component = new SectionComponent($this->uuid->generate(), $region, ['id' => $block_plugin->getPluginId()]);
    $cloned_component->setConfiguration($configuration);
    $section->insertComponent($component->getWeight(), $cloned_component);
    $this->tempstoreRepository->set($section_storage);

    return $this->rebuildAndClose($section_storage);
  }

}
