<?php

namespace Drupal\lb_plus\Event;

use Drupal\Component\EventDispatcher\Event;

class AdminButtonsEvent extends Event {

  private array $section;
  private string $delta;
  private array $buttons;
  private string $storageId;
  private string $storageType;

  public function __construct(array $section, string $storage_type, string $storage_id, string $delta) {
    $this->storageType = $storage_type;
    $this->storageId = $storage_id;
    $this->section = $section;
    $this->delta = $delta;
  }

  /**
   * Get section.
   *
   * @return array
   *   Get the current section.
   */
  public function getSection(): array {
    return $this->section;
  }

  /**
   * Get buttons.
   *
   * @return array
   *   A render array of buttons.
   */
  public function getButtons(): array {
    return $this->buttons;
  }

  /**
   * Set buttons.
   *
   * @param array $buttons
   *   A render array of buttons.
   */
  public function setButtons(array $buttons): void {
    $this->buttons = $buttons;
  }

  /**
   * Get storage type.
   *
   * @return string
   *   The storage type.
   */
  public function getStorageType(): string {
    return $this->storageType;
  }

  /**
   * Get storage ID.
   *
   * @return string
   *   The storage ID.
   */
  public function getStorageId(): string {
    return $this->storageId;
  }

  /**
   * Get delta.
   *
   * @return string
   *   The section delta.
   */
  public function getDelta(): string {
    return $this->delta;
  }

}
