<?php

namespace Drupal\lb_plus\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\lb_plus\Controller\ChooseBlock;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\layout_builder\Form\OverridesEntityForm as LayoutBuilderOverridesEntityForm;

class OverridesEntityForm implements ContainerInjectionInterface {

  protected ModuleHandlerInterface $moduleHandler;
  protected ClassResolverInterface $classResolver;

  public function __construct(ModuleHandlerInterface $module_handler, ClassResolverInterface $class_resolver) {
    $this->moduleHandler = $module_handler;
    $this->classResolver = $class_resolver;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('class_resolver'),
    );
  }

  public function formAlter(&$form, FormStateInterface $form_state, $form_id) {
    $form_object = $form_state->getFormObject();
    if (!$form_object instanceof LayoutBuilderOverridesEntityForm) {
      return;
    }
    // Clean up the LB UI by hiding a few things.
    if (!empty($form['revision_information']['#access'])) {
      $form['revision_information']['#access'] = FALSE;
    }
    if (!empty($form['actions']['preview_toggle'])) {
      $form['actions']['preview_toggle']['#access'] = FALSE;
    }
    if (!empty($form['layout_builder_message'])) {
      unset($form['layout_builder_message']);
    }
    // Style the form actions as a toolbar.
    $form['actions']['#attributes']['class'][] = 'lb-plus-toolbar';
    $form['actions']['#attributes']['data-offset-bottom'] = '';
    $form['actions']['toggle-add-block-sidebar'] = [
      '#type' => 'container',
      '#weight' => 999999999,
      '#attributes' => [
        'title' => t('Add block'),
        'id' => 'lb-plus-toggle-add-block-sidebar',
        'class' => ['lb-plus-icon'],
        'style' => [
          'background-image: url("/' . $this->moduleHandler->getModule('lb_plus')->getPath() . '/assets/plus.svg");',
        ],
      ],
    ];
    // Include a side bar for adding blocks to the layout.
    $section_storage = $form_object->getSectionStorage();
    $form['sidebar_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'lb-plus-add-block'],
      'sidebar' => $this->classResolver->getInstanceFromDefinition(ChooseBlock::class)->chooseBlockDialog($section_storage->getStorageId(), $section_storage),
    ];
  }

}
