<?php

namespace Drupal\lb_plus;

use Drupal\layout_builder\SectionStorageInterface;

/**
 * Trait for retrieving LB+ settings.
 */
trait LayoutBuilderPlusSettingsTrait {

  /**
   * Get a Layout Builder + setting.
   *
   * Gets a layout builder + setting from the entity_view_display third
   * party settings.
   *
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   * @param string $setting
   *   The setting key to retrieve.
   *
   * @return mixed
   *   The setting.
   *
   * @throws \Drupal\Component\Plugin\Exception\ContextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getLbPlusSetting(SectionStorageInterface $section_storage, string $setting) {
    $view_mode = $section_storage->getContext('view_mode')->getContextValue();
    $entity = $section_storage->getContext('entity')->getContextValue();
    $entity_view_display_id = sprintf('%s.%s.%s', $entity->getEntityTypeId(), $entity->bundle(), $view_mode);
    $entity_view_display = $this->entityTypeManager()->getStorage('entity_view_display')->load($entity_view_display_id);
    return $entity_view_display->getThirdPartySetting('lb_plus', $setting);
  }

}
